-- pNameplates
-- Version:  1.01
-- Authors:  Crushbeers, Snìffle

-- TODO: Tank Threat Option

--[[       TABLE OF CONTENTS      ]]--
--[[ ---------------------------- ]]--
--[[ 0. DEFAULT VARIABLE SETTINGS ]]--
--[[ 1. TEXTURES/LOCAL VARIABLES  ]]--
--[[ 2. UTILITY FUNCTIONS		  ]]--
--[[ 3. STYLE FUNCTIONS			  ]]--
--[[ 4. THREAT MANAGEMENT		  ]]--
--[[ 5. UPDATE FUNCTIONS		  ]]--
--[[ 6. CORE FUNCTIONS			  ]]--
--[[ 7. SLASH COMMANDS			  ]]--

--------------------------------------
-- 0. DEFAULT VARIABLE SETTINGS
--------------------------------------

ShowPercent = true
ShowWhenMax = true
ThreatEnabled = true
DPSHealerMode = true
FontSize = 12
FontType = "Fonts\\ARIALN.ttf"

--------------------------------------
-- 1. TEXUTURES AND LOCAL VARIABLES
--------------------------------------

local Version = "1.01"

local total        = -1
local ForceUpdate  = false
local InCombat     = false
local PVPCombat    = false

local backgroundTexture  	  = "Interface\\Buttons\\WHITE8x8"
local borderDefault      	  = "Interface\\Tooltips\\Nameplate-Border"
local borderTexture      	  = "Interface\\AddOns\\pNameplates\\Textures\\border"
local barTexture         	  = "Interface\\AddOns\\pNameplates\\Textures\\bar"
local React                   = {}
local Players                 = {}

-- Default Unit Colors:
local Colors = {
 -- Key -- { Red, Green, Blue }
	[1]  = { 1.00, 0.00, 0.00 }, -- Hostile
	[2]  = { 1.00, 1.00, 0.00 }, -- Neutral
	[3]  = { 0.00, 1.00, 0.00 }, -- Friendly NPC
	[4]  = { 0.77, 0.12, 0.23 }, -- DK
	[5]  = { 1.00, 0.49, 0.04 }, -- Druid
	[6]  = { 0.67, 0.83, 0.45 }, -- Hunter
	[7]  = { 0.41, 0.80, 0.94 }, -- Mage
	[8]  = { 0.33, 0.54, 0.52 }, -- Monk
	[9]  = { 0.96, 0.55, 0.73 }, -- Paladin
	[10] = { 1.00, 1.00, 1.00 }, -- Priest
	[11] = { 1.00, 0.96, 0.41 }, -- Rogue
	[12] = { 0.00, 0.44, 0.87 }, -- Shaman
	[13] = { 0.58, 0.51, 0.79 }, -- Warlock
	[14] = { 0.78, 0.61, 0.43 }, -- Warrior
	[15] = { 0.00, 0.00, 1.00 }  -- Friendly Player
}

-- Font Variables
local Fonts = {
	["FontType1"] = "ARIALN",
	["FontType2"] = "FRIZQT__",
	["FontType3"] = "MORPHEUS",
	["FontType4"] = "skurri",
}

-- Threat Setting Lists
local ThreatLow = {
	Right   = { loc = "TOPRIGHT", x = 1, y = 2 },
	Left    = { loc = "BOTTOMLEFT", x = -1, y = -2 },
	Color   = { r = 0.0, g = 0.7, b = 0.0 },
	Texture = "Interface\\AddOns\\pNameplates\\Textures\\threat_LOW",
	HPScale = 1,
}

local ThreatMed = {
	Right   = { loc = "TOPRIGHT", x = 4, y = 5 },
	Left    = { loc = "BOTTOMLEFT", x = -4, y = -5 },
	Color   = { r = 0.7, g = 0.7, b = 0.0 },
	Texture = "Interface\\AddOns\\pNameplates\\Textures\\threat_MED",
	HPScale = 1,
}

local ThreatHigh = {
	Right   = { loc = "TOPRIGHT", x = 4, y = 5 },
	Left    = { loc = "BOTTOMLEFT", x = -4, y = -5 },
	Color   = { r = 0.8, g = 0.0, b = 0.0 },
	Texture = "Interface\\AddOns\\pNameplates\\Textures\\threat_HIGH",
	HPScale = 1.18,
}

--------------------------------------
-- 2. UTILITY FUNCTIONS
--------------------------------------

local function RoundNum(num, dec)
	num = num * math.pow(10, dec)
	num = math.ceil(num)
	num = num / math.pow(10, dec)
	return num
end

local function AlignObject(object)
	local _, anchor, _, x, y = object:GetPoint()
	object:ClearAllPoints()
	object:SetPoint("BOTTOM", anchor, "BOTTOM", x, y)
end

local function IsNameplate(object)
    local bo = select(2, object:GetRegions())
    if (bo and bo:GetTexture() == borderTexture) then return true end
    if (not bo or bo:GetObjectType() ~= "Texture" or bo:GetTexture() ~= borderDefault) then
      	object.Styled = true
      	return false
    end
    return true
end

--------------------------------------
-- 3. STYLE FUNCTIONS
--------------------------------------

local function StyleTextures(plate)
	if (not plate.Highlight or not plate.Threat) then return end
	plate.Highlight:SetTexture(1, 1, 1, 0.2)
	plate.Highlight:SetPoint("TOPLEFT", plate.Border, "TOPLEFT", 1, -1)
	plate.Highlight:SetPoint("BOTTOMRIGHT", plate.Border, "BOTTOMRIGHT", -1, 1)

	plate.Threat:ClearAllPoints()
	plate.Threat:SetPoint("CENTER", plate.Border, "CENTER", 8, -3)
end

local function HideBlizz(plate)
	plate.Name:Hide()
	plate.Threat:SetTexture(0, 0, 0, 0)
	plate.Dragon:SetTexture(0, 0, 0, 0)
    plate.Castbar.Shield:SetTexture(0, 0, 0, 0)
end

local function StyleCastbar(plate)
	plate.Castbar:SetBackdrop({
        bgFile = backgroundTexture,
        insets = { left = -1, right = -1, top = -1, bottom = -1 }
    })
    plate.Castbar:SetBackdropColor(0, 0, 0, 0.6)
    plate.Castbar:ClearAllPoints()
    plate.Castbar:SetPoint("TOPRIGHT", plate.Health, "BOTTOMRIGHT", 0, -4)
    plate.Castbar:SetPoint("BOTTOMLEFT", plate.Health, "BOTTOMLEFT", 0, -15)
	
	plate.Castbar.Border:SetTexture(0, 0, 0, 1)
    plate.Castbar.Border:SetTexCoord(0, 1, 0, 1)
    plate.Castbar.Border:ClearAllPoints()
    plate.Castbar.Border:SetPoint("TOPRIGHT", plate.Castbar, 0, 0)
    plate.Castbar.Border:SetPoint("BOTTOMLEFT", plate.Castbar, 0, 0)
    plate.Castbar.Border:SetDrawLayer("BORDER")
    
    plate.Castbar.Fill:SetTexture(barTexture)
    plate.Castbar.Fill:ClearAllPoints()
    plate.Castbar.Fill:SetPoint("TOPRIGHT", plate.Castbar.Border, "TOPRIGHT", 1, -1)
    plate.Castbar.Fill:SetPoint("TOPRIGHT", plate.Castbar.Border, "BOTTOMLEFT", -1, 1)
    
    plate.Castbar.Icon:SetSize(12,12)
    plate.Castbar.Icon:ClearAllPoints()
    plate.Castbar.Icon:SetPoint("LEFT", plate.Castbar, "RIGHT", 2, 0)
    
    plate.Castbar.Time = plate.Castbar:CreateFontString(nil, "HIGH")
	plate.Castbar.Time:SetFont(FontType, 10, "THINOUTLINE")
	plate.Castbar.Time:SetPoint("RIGHT", 0, 0)
	
	plate.Castbar.Spell = plate.Castbar:CreateFontString(nil, "HIGH")
	plate.Castbar.Spell:SetFont(FontType, 10, "THINOUTLINE")
	plate.Castbar.Spell:SetPoint("LEFT", 1, 0)
end

local function HealthTextStyle(plate)
	plate.Health.hpval:SetFont(FontType, FontSize, "THINOUTLINE")
end

--------------------------------------
-- 4. THREAT MANAGEMENT
--------------------------------------

local function AssignDefaultColor(plate)
	local r = RoundNum(select(1, plate.Health:GetStatusBarColor()), 2)
	local g = RoundNum(select(2, plate.Health:GetStatusBarColor()), 2)
	local b = RoundNum(select(3, plate.Health:GetStatusBarColor()), 2)
	
	Players[plate.Text] = false -- Initialize to false
	
	for i,v in ipairs(Colors) do
		if (r == v[1] and g == v[2] and b == v[3]) then
			React[plate.Text] = v
			if (i > 3) then Players[plate.Text] = true end
			break
		end
	end
end

local function SetThreatEnabled(plate, Threat)	
	plate.ThreatBorder:SetPoint(Threat.Right.loc, plate.Border, Threat.Right.x, Threat.Right.y)
	plate.ThreatBorder:SetPoint(Threat.Left.loc, plate.Border, Threat.Left.x, Threat.Left.y)
	plate.ThreatBorder:SetTexture(Threat.Texture)
	plate.Health:SetStatusBarColor(Threat.Color.r, Threat.Color.g, Threat.Color.b, 1)
	plate.Health:SetScale(Threat.HPScale)
end

local function UpdateThreat(plate)
	if (Players[plate.Text] == nil) then AssignDefaultColor(plate) end
	if (not InCombat and plate.ThreatBorder:IsShown()) or (Players[plate.Text]) then
		plate.ThreatBorder:Hide()
		plate.Health:SetStatusBarColor(React[plate.Text][1], React[plate.Text][2], React[plate.Text][3], 1)
		plate.Health:SetScale(1)
		return 
	elseif (InCombat) then
		plate.ThreatBorder:Show()
	else return end -- Returns if out of combat AND threat objects are hidden
	
	local red, green, blue, _ = plate.Threat:GetVertexColor()
	local shown = plate.Threat:IsShown()
	
	if (not shown) then 
		if (DPSHealerMode) then SetThreatEnabled(plate, ThreatLow)
		else SetThreatEnabled(plate, ThreatHigh) end
	elseif (red > 0) then
		if (green > 0) then SetThreatEnabled(plate, ThreatMed) return end
		if (DPSHealerMode) then SetThreatEnabled(plate, ThreatHigh)
		else SetThreatEnabled(plate, ThreatLow) end
	end
end

--------------------------------------
-- 5. UPDATE FUNCTIONS
--------------------------------------

local function UpdateDisplay(plate)
	plate.Text = plate.Name:GetText()
	if (ThreatEnabled) then UpdateThreat(plate) end
	
	plate.Health:SetHeight(14)
	plate.NewName:SetText(plate.Text)
	
	if (ShowPercent) then
		local _, max = plate.Health:GetMinMaxValues()
		local value = plate.Health:GetValue()
		local percent = math.floor((value/max)*100)
		if (percent >= 100 and not ShowWhenMax) then plate.Health.hpval:SetText("")
		else plate.Health.hpval:SetText(percent.."%") end
	else
		plate.Health.hpval:SetText("")
	end
end

local function UpdateCastInfo(Castbar)
	local SpellName = select(3, UnitCastingInfo("target"))
	if (not SpellName) then SpellName = select(3, UnitChannelInfo("target")) end
	local _, max = Castbar:GetMinMaxValues()
	local current = Castbar:GetValue()
	local final = math.floor((max-current)*10)/10
	
	Castbar.Time:SetText(final)
	Castbar.Spell:SetText(SpellName)
	
	-- Fixes phantom castbar frames appearing
	if (not final or final <= 0) then Castbar:Hide()
	else Castbar:Show() end
end

--------------------------------------
-- 6. CORE FUNCTIONS
--------------------------------------

local function init(plate)
	plate:SetToplevel(true)

	plate.Health, plate.Castbar = plate:GetChildren()
    plate.Castbar.Fill, plate.Castbar.Border, plate.Castbar.Shield, plate.Castbar.Icon = plate.Castbar:GetRegions()
    plate.Threat, plate.Border, plate.Highlight, plate.Name, plate.Level, plate.Boss, _, plate.Dragon = plate:GetRegions()

	plate.Border:SetTexture(borderTexture)
    plate.Border:SetVertexColor(0, 0, 0)
    plate.Border:ClearAllPoints()
    plate.Border:SetPoint("TOPRIGHT", plate.Health, 1, 1)
    plate.Border:SetPoint("BOTTOMLEFT", plate.Health, -1, -1)
    
    plate.ThreatBorder = plate:CreateTexture(nil, "Texture")
    plate.ThreatBorder:SetPoint("TOPRIGHT", plate.Border, 1, 2)
    plate.ThreatBorder:SetPoint("BOTTOMLEFT", plate.Border, -1, -2)
    
	plate.Health:SetHeight(14)
	plate.Health:SetStatusBarTexture(barTexture)
    plate.Health:SetBackdrop({
        bgFile = backgroundTexture,
        insets = { left = 0, right = 0, top = 0, bottom = 0 }
    })
    plate.Health:SetBackdropColor(0, 0, 0, 0.6)
	
	plate.Level:ClearAllPoints()
	plate.Level:SetPoint("LEFT", plate.Health, "RIGHT", 2, 0)
	plate.Level:SetFont(FontType, 12, "THINOUTLINE")
	plate.Level:SetShadowColor(0 ,0, 0, 0)
	
	plate.Boss:ClearAllPoints()
	plate.Boss:SetPoint(plate.Level:GetPoint())
	
	plate.NewName = plate:CreateFontString(nil, "HIGH")
	plate.NewName:SetPoint("BOTTOM", plate.Health, "BOTTOM", 0, 2)
	plate.NewName:SetFont(FontType, 10, "NONE")
	plate.NewName:SetShadowColor(0, 0, 0, 1)
	plate.NewName:SetShadowOffset(1, -1)
	plate.NewName:SetText(plate.Name:GetText())
	plate.NewName:SetJustifyH("CENTER")
	plate.NewName:SetWordWrap(false)
	plate.NewName:SetWidth(100)
	
	plate.Health.hpval = plate.Health:CreateFontString(nil, "HIGH")
	plate.Health.hpval:SetFont(FontType, FontSize, "THINOUTLINE")
	plate.Health.hpval:SetPoint("BOTTOM", plate.Health, "TOP", 0, 2)
	plate.Health.hpval:SetJustifyH("CENTER")
	
	plate.Styled = true
	HideBlizz(plate)
	UpdateDisplay(plate)
	StyleTextures(plate)
	AlignObject(plate.Health)
	StyleCastbar(plate)
	
	plate.Castbar:SetScript("OnValueChanged", UpdateCastInfo)
	
	plate.Health:SetScript("OnShow", function()
		UpdateDisplay(plate)
		StyleTextures(plate)
		UpdateCastInfo(plate.Castbar)
		AlignObject(plate.Health)
	end)
	plate.Health:SetScript("OnHide", function()
		UpdateDisplay(plate)
	end)
	plate.Health:SetScript("OnValueChanged", function()
		UpdateDisplay(plate)
	end)
	plate.Health:SetScript("OnMinMaxChanged", function()
		UpdateDisplay(plate)
	end)
end

local function GetObjects(plate, elapsed)
    local current = select("#", WorldFrame:GetChildren())
    if (current ~= total or ForceUpdate) then
    	total = current
	  	for i = 1, current do
			local plate = select(i, WorldFrame:GetChildren())
			if (IsNameplate(plate)) then
		  		if (not plate.Styled) then init(plate)
		  		else 
		  			UpdateDisplay(plate)
		  			StyleTextures(plate)
		  			if (ForceUpdate) then HealthTextStyle(plate) end
		  		end
		  	end
		end
		ForceUpdate = false
    end
end

local f = CreateFrame("Frame")
f:SetScript("OnEvent", function(plate, event)
	ForceUpdate = true
	if (event == "PLAYER_LOGIN") then
		SetCVar("ShowClassColorInNameplate", 1)
		plate:SetScript("OnUpdate", GetObjects)
	elseif (event == "PLAYER_REGEN_DISABLED") then
		InCombat = true
	elseif (event == "PLAYER_REGEN_ENABLED") then
		InCombat = false
	end
end)
  
f:RegisterEvent("PLAYER_LOGIN")
f:RegisterEvent("PLAYER_REGEN_DISABLED") -- Enter Combat
f:RegisterEvent("PLAYER_REGEN_ENABLED")  -- Leave Combat

--------------------------------------
-- 7. SLASH COMMANDS
--------------------------------------

local function handler(args)
	local cmd = string.split(' ', args):lower() or args:lower()
	if (cmd:match"font") then
		cmd = select(2, string.split(' ', args))
		if (not cmd) then cmd = "showcommands" end
		if (cmd:match"size") then
			local TempSize = (select(3, string.split(' ', args)) + 0)
			if (type(TempSize) == "number") then 
				FontSize = TempSize
				print("|cFF6EE1E6p|rNameplates |cFF6EE1E6Font size set to|r "..FontSize.."|cFF6EE1E6.|r")
			end
		elseif (cmd:match"type") then
			FontType = "Fonts\\"..Fonts["FontType"..select(3, string.split(' ', args))]..".ttf"
			local change = Fonts["FontType"..select(3, string.split(' ', args))]
    		print("|cFF6EE1E6p|rNameplates |cFF6EE1E6Font type changed to |r"..change.."|cFF6EE1E6.|r")
		else
			print("|cFF6EE1E6p|rNameplates |cFF6EE1E6Font commands:|r")
			print("|cFF6EE1E6/pp font type #|r, (1-4) to select FontType.")
			print("|cFF6EE1E6/pp font size ##|r, to set percent FontSize.")
		end
    elseif (cmd:match"percent") then
    	if (ShowPercent) then
			ShowPercent = false
			print("|cFF6EE1E6p|rNameplates|cFF6EE1E6: Health percent disabled.|r")
		else
			ShowPercent = true
			print("|cFF6EE1E6p|rNameplates|cFF6EE1E6: Health percent enabled.|r")
		end
    elseif (cmd:match"max") then
    	if (ShowWhenMax) then 
    		ShowWhenMax = false
    		print("|cFF6EE1E6p|rNameplates|cFF6EE1E6: 100% Health display disabled.|r")
    	else
    		ShowWhenMax = true
    		print("|cFF6EE1E6p|rNameplates|cFF6EE1E6: 100% Health display enabled.|r")
    	end
    elseif (cmd:match"threat") then
    	if (ThreatEnabled) then 
    		ThreatEnabled = false
    		print("|cFF6EE1E6p|rNameplates|cFF6EE1E6: Threat coloring disabled.|r")
    	else
    		ThreatEnabled = true
    		print("|cFF6EE1E6p|rNameplates|cFF6EE1E6: Threat coloring enabled.|r")
    	end
    elseif (cmd:match"role") then
    	cmd = select(2, string.split(' ', args))
    	if (not cmd) then cmd = "showcommands" end
    	if (cmd:match"tank") then
    		DPSHealerMode = false
    		print("|cFF6EE1E6p|rNameplates|cFF6EE1E6: Tank mode enabled.|r")
    	elseif (cmd:match"dps" or cmd:match"heal") then
    		DPSHealerMode = true
    		print("|cFF6EE1E6p|rNameplates|cFF6EE1E6: DPS/Healing mode enabled.|r")
    	else
    		print("|cFF6EE1E6p|rNameplates |cFF6EE1E6Role commands:|r")
    		print("|cFF6EE1E6/pp role tank|r, for Tank mode.")
    		print("|cFF6EE1E6/pp role dps OR heal|r, for DPS/Healing mode.")
    	end
    elseif (cmd:match"motion") then
    	cmd = select(2, string.split(' ', args))
    	if (not cmd) then cmd = "showcommands" end
    	if (cmd:match"0") then SetCVar("nameplateMotion", 0)
    	elseif (cmd:match"1") then SetCVar("nameplateMotion", 1)
    	elseif (cmd:match"2") then SetCVar("nameplateMotion", 2)
    	else
    		print("|cFF6EE1E6p|rNameplates |cFF6EE1E6Motion commands:|r")
    		print("|cFF6EE1E6/pp motion 0|r, for Overlap.")
    		print("|cFF6EE1E6/pp motion 1|r, for Stacking.")
    		print("|cFF6EE1E6/pp motion 2|r, for Spreading.")
    	end
    else
		print("|cFF6EE1E6p|rNameplates |cFF6EE1E6command list:|r")
		print("|cFF6EE1E6/pp percent|r, to toggle percents.")
		print("|cFF6EE1E6/pp max|r, to toggle 100% Health.")
		print("|cFF6EE1E6/pp font|r, to see font options.")
		print("|cFF6EE1E6/pp role|r, to see role options.")
		print("|cFF6EE1E6/pp threat|r, to toggle Threat coloring.")
		print("|cFF6EE1E6/pp motion|r, to see nameplate motion options.")
    end
    ForceUpdate = true
end
 
SLASH_pp1 = "/pp";
SlashCmdList["pp"] = handler;

print("|cFF6EE1E6p|rNameplates  |cFF6EE1E6(v"..Version..")|r  is loaded: |cFF6EE1E6/pp|r for help.")